SUBROUTINE disort_wrapper(Nz,g0,i_bottom,i_top,tau,Ttab,Ztab,Tsurf,Ttop, &
     rho_v, rho_c, rho_o, cloud_flag, tau_cloud, sig_deb, sig_fin, &
     flux_up, dflux_dtau, kext_top)

  implicit none
  
  integer, intent(in) :: Nz, i_bottom, i_top, cloud_flag ! 1=Earth 0=None, 2=Venus
  double precision, intent(in) :: g0, sig_deb, sig_fin
  double precision, intent(in) :: Tsurf, Ttop
  double precision, intent(in), dimension(0:Nz) :: rho_v, rho_c, rho_o, Ttab, Ztab
  double precision, intent(in), dimension(Nz) :: tau
  
  double precision, intent(out) :: flux_up, dflux_dtau, kext_top
  real, intent(out) :: tau_cloud

  
  ! DISORT variables
  integer, parameter :: maxcly=1600,maxulv=maxcly+1,nerr=22,maxmom=4,maxumu=1, maxphi=1
  integer :: ibcnd, nlyr, ntau, nmom, nstr, numu, nphi
  character*127 :: header
  logical :: planck, usrtau, usrang, lamber, onlyfl
  logical, dimension(5) :: prnt
  real :: albedo, btemp, fbeam, fisot, temis, ttemp, wvnmhi, wvnmlo, umu0, phi0, accur, pi
  real, dimension(0:maxcly) :: temper
  real, dimension(maxcly) :: dtauc, gg, ssalb
  real, dimension(0:maxmom,maxcly) :: pmom
  real, dimension(maxulv) :: utau,dfdt,flup,rfldir,rfldn,uavg
  real, dimension(maxumu) :: umu, albmed, trnmed
  real, dimension(maxphi) :: phi
  real, dimension(maxumu,maxulv,maxphi) :: uu

  ! other internal variables
  
  real :: Qext,g,w0,Mw,dtau_cloud,hscale,Dz,factor,mload,lambda
  integer :: imid, iz, imom
  
  header  = "DISORT"
  prnt(:) = .FALSE.
  nlyr    = Nz
  usrtau  = .TRUE.
  ntau    = 1
  utau(1) = 0.0
  albedo  = 0.0
  fisot   = 0.0
  planck  = .TRUE.
  fbeam   = 0.
  umu0    = 0.

  lambda = 2e4/(sig_deb+sig_fin) ! -> micron
  
  DO iz=1,nlyr
     dtauc(iz)  = tau(Nz-iz+1)
     ssalb(iz)  = 0.
     gg(iz)     = 0.
     temper(iz) = Ttab(Nz-iz)
  END DO
  
  tau_cloud = 0.

  SELECT CASE (cloud_flag)
  CASE (1) ! Earth
     mload = 4e-4*130.
     IF (lambda < 2.) THEN
        Qext = 1.
        g    = 0.85
        w0   = 1.
     ELSE IF ((lambda >= 2.) .AND. (lambda < 10.)) THEN
        Qext = 1.
        g    = 0.85
        w0   = 1.24*lambda**(-.32)
     ELSE IF ((lambda >= 10.) .AND. (lambda < 20.)) THEN
        Qext = 1.
        g    = 1.4*lambda**(-.22)
        w0   = 1.24*lambda**(-.32)
     ELSE IF (lambda > 20.) THEN
        Qext = 3.26*lambda**(-.4)
        g    = 1.4*lambda**(-.32)
        w0   = 1.24*lambda**(-.32)
     END IF
  CASE (2) ! Venus
     mload = 1e-5*407.5
     w0    = min(1.,exp(1.16-1.42*log(lambda)-0.05*log(lambda)**2))
     IF (lambda < 3.37) THEN
        Qext = 1.
        g    = 0.73
     ELSE IF ((lambda >= 3.37).AND.(lambda < 5.63)) THEN
        Qext = 1.
        g    = 7.28*lambda**(-1.89)
     ELSE IF (lambda >= 5.63) THEN
        Qext = 10.86*lambda**(-1.38)
        g    = 7.28*lambda**(-1.89)
     END IF
  END SELECT
  
  clouds: IF (cloud_flag /= 0) THEN
     open(71,file='results/clouds.txt')
     write(71,*) 'lambda = ',lambda
     write(71,*) 'w0 clouds = ',w0
     write(71,*) 'g = ',g
     write(71,*) 'Qext = ',Qext
     
     write(71,*) 'iz   Z  dtau_c   dtau    w0   factor'
     
     imid = (i_bottom+i_top)/2
     mw = (rho_c(imid)*44d-3+rho_v(imid)*18d-3+rho_o(imid)*28d-3)/ &
          (rho_c(imid)+rho_v(imid)+rho_o(imid))
     hscale = 8.314*Ttab(imid)/(mw*g0)
     
     DO iz=Nz-i_top+1,Nz-i_bottom+1
        Dz = Ztab(Nz-iz+1)-Ztab(Nz-iz)
        
        factor = min(1., &
             exp(-.5+.5*Dz/hscale*(iz-Nz+i_top-1)), &
             exp(-.5+.5*Dz/hscale*(Nz-i_bottom+1-iz)) &
             )
        ! factor = 1.
        gg(iz) = g
        dtau_cloud = mload*(rho_o(Nz-iz)+rho_v(Nz-iz)+rho_c(Nz-iz))* &
             Qext*Dz*factor
        dtauc(iz)  = dtauc(iz)+dtau_cloud
        tau_cloud  = tau_cloud+dtau_cloud
        ssalb(iz)  = w0*dtau_cloud/dtauc(iz)
        write(71,*) Nz-iz, Ztab(Nz-iz), dtau_cloud, dtauc(iz), ssalb(iz), factor
     END DO
     write(71,*) 'Total cloud opacity = ',tau_cloud
     close(71)
  END IF clouds

  wvnmlo    = sig_deb
  wvnmhi    = sig_fin
  btemp     = Tsurf
  ttemp     = Ttop
  temper(0) = Ttab(Nz)
  temis     = 0.
  
  nmom = 4
  
  DO imom=0,nmom
     pmom(imom,:) = gg(:)**imom ! Henyey-Greenstein phase function
  END DO
  
  nstr = 4
  
  usrang = .TRUE.
  
  numu = 0
  nphi = 0
  
  umu(1) = 0.999
  phi(1) = 0.
  
  ibcnd = 0
  
  phi0 = 0
  
  lamber = .TRUE.
  onlyfl = .TRUE.
  
  accur = 5e-3

  CALL DISORT( nlyr, dtauc, ssalb, nmom, pmom, temper, wvnmlo, &
       wvnmhi, usrtau, ntau, utau, nstr, usrang, numu, umu, &
       nphi, phi, ibcnd, fbeam, umu0, phi0, fisot, lamber,&
       albedo, btemp, ttemp, temis, planck, onlyfl, accur, &
       prnt, header, maxcly, maxulv, maxumu, maxphi, maxmom, &
       rfldir, rfldn, flup, dfdt, uavg, uu, albmed, trnmed )
  
  flux_up = flup(1)
  dflux_dtau = dfdt(1)
  kext_top = sum(dtauc(1:Nz-i_top))/(Ztab(Nz)-Ztab(i_top)) ! meso avg
  
END SUBROUTINE DISORT_WRAPPER
