SUBROUTINE make_tau_grey(Nz,Ttab,Ztab,rho_v,rho_c,rho_o, &
     sig_deb,sig_fin,tau,flag_cont)

  implicit none

  integer, intent(in) :: Nz, flag_cont
  double precision, intent(in) :: sig_deb, sig_fin
  double precision, intent(in), dimension(0:Nz) :: Ttab, Ztab, rho_v, rho_c, rho_o
  double precision, intent(out), dimension(Nz) :: tau

  integer :: iz,j0,i
  double precision :: alpha
  double precision :: c_v, c_c, c_c_tab(12), c_v_tab(6), m_v, m_c, w_v, w_c, n_v, n_c
  double precision :: T_mid, Dz
  double precision :: tau_b, tau_cv, tau_cc
  
  CALL tab_cont_v(sig_deb,sig_fin,c_v_tab)
  CALL tab_cont_c(sig_deb,sig_fin,c_c_tab)

  main: DO iz=1,Nz
   
     Dz = Ztab(iz)-Ztab(iz-1) ! m
     
     T_mid = .5*(Ttab(iz-1)+Ttab(iz))
   
     T_mid = min(7d2,T_mid) ! temporary protection against overflow

     ! interpolation en T de C_n

     j0 = 1 + floor((T_mid-200.)/50.)
     j0 = min(j0,11)
     j0 = max(j0,1)
     alpha = (T_mid-(150.+50.*j0))/50.
     
     C_c = alpha*log10(C_c_tab(j0+1))+(1.-alpha)*log10(C_c_tab(j0))
     C_c = 1d1**C_c
     
     C_c = 3.723d-25*C_c

     ! interpolation en T de C_v
     j0 = 1 + floor((T_mid-200.)/100.)
     j0 = min(j0,5)
     j0 = max(j0,1)
     alpha = (T_mid-(100.+100.*j0))/100.
   
     C_v = alpha*log10(c_v_tab(j0+1))+(1.-alpha)*log10(c_v_tab(j0))
     C_v = 1d1**C_v

     T_mid = .5*(Ttab(iz-1)+Ttab(iz)) ! restoring T_mid

     m_v = Dz*(rho_v(iz-1)+rho_v(iz))*.5*1d-1 ! -> g/cm^2
     m_c = Dz*(rho_c(iz-1)+rho_c(iz))*.5*1d-1 ! -> g/cm^2

     n_v = .5*(rho_v(iz-1)+rho_v(iz))*3.3456d19 ! -> molec/cm^3
     w_v = n_v * Dz / 2.69d17 ! -> cm-atm

     n_c = .5*(rho_c(iz-1)+rho_c(iz))*1.369d19 ! -> molec/cm^3
     w_c = n_c * Dz / 2.69d17 ! -> cm-atm

     tau_cv = c_v * n_v * w_v
     tau_cc = c_c * n_c * w_c
     
     tau_b = sqrt(rho_v(iz)*rho_v(iz-1))*Dz*1d-2 & ! kappa_H2O Nakajima
            +sqrt(rho_c(iz)*rho_c(iz-1))*Dz*1d-4   ! kappa CO2 Nakajima

     SELECT CASE(flag_cont)
     CASE(3)
        tau(iz) = tau_b + tau_cv + tau_cc ! all
     CASE(2)
        tau(iz) = tau_b + tau_cv
     CASE(1)
        tau(iz) = tau_b + tau_cc
     CASE(0)
        tau(iz) = tau_b
     CASE DEFAULT
        print*, "flag_cont not recognized. Defaulting to full continuum"
        tau(iz) = tau_b + tau_cv + tau_cc
     END SELECT
   
  END DO main

END SUBROUTINE MAKE_TAU_GREY

SUBROUTINE tab_cont_v(s_deb,s_fin,c_v_tab)

  implicit none

  double precision, intent(in) :: s_deb, s_fin
  double precision, intent(out), dimension(6) :: c_v_tab ! 1/(cm^2 atm)

  integer, parameter :: Ntab=1000
  integer :: i, i_deb, i_fin, it
  double precision, dimension(6,Ntab) :: c_v_tab2
  double precision, dimension(Ntab) :: sig_tab
  double precision s,a,b,c,d
  character*1 :: str_it
  character*25 :: filestr

  DO it=1,6
     write(str_it,'(I1)') it+1
     filestr = 'kspectrum/WATER.COEF.'//str_it//'00K'

     open(61,file=filestr)
     ! syntaxe : sigma wrs, wrf, rs, rf. On veut rs

     DO i=1,Ntab
        read(61,*) s, a, b, c, d
        sig_tab(i) = s
        c_v_tab2(it,i) = c
     END DO
     c_v_tab2(it,1) = c_v_tab2(it,2) ! protection anti-zero
     
     close(61)
  END DO

  i_deb = 1
  i_fin = 1
  DO i=1,Ntab-1
     IF (sig_tab(i) < s_deb) THEN
        i_deb = i_deb+1
     END IF
     IF (sig_tab(i) < s_fin) THEN
        i_fin = i_fin+1
     END IF
  END DO

  DO it=1,6
     c_v_tab(it)=0.
     DO i=i_deb,i_fin
        c_v_tab(it)=c_v_tab(it)+dlog10(c_v_tab2(it,i))/(i_fin-i_deb+1.)
     END DO
     c_v_tab(it) = 1d1**c_v_tab(it)
  END DO

END SUBROUTINE TAB_CONT_V

SUBROUTINE tab_cont_c(s_deb,s_fin,c_c_tab)
  implicit none

  double precision, intent(in) :: s_deb, s_fin
  double precision, intent(out), dimension(12) :: c_c_tab ! 1/(cm^2 atm)

  integer, parameter :: Ntab2 = 112
  integer :: i,j,i_deb,i_fin,j_deb
  double precision, dimension(Ntab2) :: sig_tab2
  double precision, dimension(12) :: T_tab, arr
  double precision, dimension(12,Ntab2) :: c_c_tab2
  double precision :: alpha, s, a, b, c, d

  open(62,file='kspectrum/co2_cont.txt')
  read(62,*) T_tab
  
  DO i=1,Ntab2
     read(62,*) s, arr
     sig_tab2(i) = s
     DO j=1,12
        c_c_tab2(j,i) = arr(j) ! en 1e-5*invcm*am^-2
        IF (sig_tab2(i) < 4000.) THEN
           c_c_tab2(j,i) = max(c_c_tab2(j,i),3.5d-3)
        END IF
     END DO
  END DO

  close(62)

  i_deb = 1
  i_fin = 1

  DO i=1,Ntab2-1
     IF (sig_tab2(i) < s_deb) THEN
        i_deb = i_deb+1
     END IF
     IF (sig_tab2(i) < s_fin) THEN
        i_fin = i_fin+1
     END IF
  END DO

  DO j=1,12
     c_c_tab(j)=0.
     
     DO i=i_deb,i_fin
        c_c_tab(j)=c_c_tab(j)+log10(c_c_tab2(j,i))/(i_fin-i_deb+1.)
     END DO
     c_c_tab(j) = 1d1**(c_c_tab(j))
  END DO

END SUBROUTINE tab_cont_c
