SUBROUTINE make_tauk(Nz,Ttab,Ztab,rho_v,rho_c,rho_o,Pv,Pc,Po, &
     iT,iP,iQ,kdist_g,Tarr,Parr,Qarr,sig_deb,sig_fin,tau,flag_cont)

implicit none

integer, intent(in) :: Nz, flag_cont, iT, iP, iQ
double precision, intent(in) :: sig_deb, sig_fin
double precision, intent(in), dimension(0:Nz) :: Ttab, Ztab, rho_v, rho_c, rho_o
double precision, intent(in), dimension(0:Nz) :: Pv, Pc, Po
double precision, intent(in), dimension(iT,iP,iQ) :: kdist_g
real, intent(in) :: Tarr(iT), Parr(iP), Qarr(iQ)
double precision, intent(out), dimension(Nz) :: tau

integer, parameter :: Nq=16
integer :: iz,j0,i
real :: garr(1:Nq)
double precision :: alpha
double precision :: c_v, c_c, c_c_tab(12), c_v_tab(6), m_v, m_c, w_v, w_c, n_v, n_c
double precision :: T_mid, P_mid, Q_mid, Dz
double precision :: tau_b, tau_cv, tau_cc

CALL tab_cont_v(sig_deb,sig_fin,c_v_tab)
CALL tab_cont_c(sig_deb,sig_fin,c_c_tab)

main: DO iz=1,Nz
   
   Dz = Ztab(iz)-Ztab(iz-1) ! m

   T_mid = .5*(Ttab(iz-1)+Ttab(iz))

   T_mid = min(7d2,T_mid) ! temporary protection against overflow

   ! interpolation en T de C_n

   j0 = 1 + floor((T_mid-200.)/50.)
   j0 = min(j0,11)
   j0 = max(j0,1)
   alpha = (T_mid-(150.+50.*j0))/50.

   C_c = alpha*dlog10(C_c_tab(j0+1))+(1.-alpha)*dlog10(C_c_tab(j0))
   C_c = 1d1**C_c

   C_c = 3.723d-25*C_c

   ! interpolation en T de C_v
   j0 = 1 + floor((T_mid-200.)/100.)
   j0 = min(j0,5)
   j0 = max(j0,1)
   alpha = (T_mid-(100.+100.*j0))/100.
   
   C_v = alpha*dlog10(c_v_tab(j0+1))+(1.-alpha)*dlog10(c_v_tab(j0))
   C_v = 1d1**C_v

   T_mid = .5*(Ttab(iz-1)+Ttab(iz)) ! restoring T_mid

   m_v = Dz*(rho_v(iz-1)+rho_v(iz))*.5*1d-1 ! -> g/cm^2
   m_c = Dz*(rho_c(iz-1)+rho_c(iz))*.5*1d-1 ! -> g/cm^2

   n_v = .5*(rho_v(iz-1)+rho_v(iz))*3.3456d19 ! -> molec/cm^3
   w_v = n_v * Dz / 2.69d17 ! -> cm-atm

   n_c = .5*(rho_c(iz-1)+rho_c(iz))*1.369d19 ! -> molec/cm^3
   w_c = n_c * Dz / 2.69d17 ! -> cm-atm

   tau_cv = c_v * n_v * w_v
   tau_cc = c_c * n_c * w_c

   P_mid = .5*(Po(iz-1)+Pc(iz-1)+Pv(iz-1)+Po(iz)+Pc(iz)+Pv(iz))
   Q_mid = (rho_v(iz)+rho_v(iz-1))/ &
           (rho_v(iz)+rho_v(iz-1)+rho_c(iz)+rho_c(iz-1))

   CALL k_correle(T_mid,P_mid,Q_mid,n_v,n_c,0d0,Dz, &
                  iT,iP,iQ,Tarr,Parr,Qarr,garr, &
                  kdist_g,tau_b)

   SELECT CASE(flag_cont)
      CASE(3)
         tau(iz) = tau_b + tau_cv + tau_cc ! all
      CASE(2)
         tau(iz) = tau_b + tau_cv
      CASE(1)
         tau(iz) = tau_b + tau_cc
      CASE(0)
         tau(iz) = tau_b
      CASE DEFAULT
         print*, "flag_cont not recognized. Defaulting to full continuum"
         tau(iz) = tau_b + tau_cv + tau_cc
      END SELECT

   END DO main

END SUBROUTINE MAKE_TAUK
